----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:33:00 10/12/2017 
-- Design Name: 
-- Module Name:    fsm - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity fsm is
    Port ( rdwr : in  STD_LOGIC;
           match : in  STD_LOGIC;
           vbit : in  STD_LOGIC;
           dbit : in  STD_LOGIC;
           readybit : in  STD_LOGIC;
			  clk : in  STD_LOGIC;
			  rst :in STD_LOGIC;
			  addressin: in std_logic_vector(16 downto 0);
------------------------------------------------
           muxSel : out  STD_LOGIC;
           demuxSel : out  STD_LOGIC;
           vbitOut : out  STD_LOGIC;
           dbitOut : out  STD_LOGIC;
			  TagEnable : out STD_LOGIC; 
			  DEnable   : out STD_LOGIC; 
			  VEnable : out STD_LOGIC; 
			  wenCache : out STD_LOGIC; 
			  cacheAddr: out std_logic_vector(7 downto 0);
			  addressOut: out std_logic_vector(15 downto 0);
			  rdwrOut : out  STD_LOGIC;
           readyOut : out  STD_LOGIC;
			  mstrb		: out  STD_LOGIC;
			  oldMuxSel : out  STD_LOGIC);
end fsm;

architecture Behavioral of fsm is

signal st1 : std_logic_vector(2 downto 0) := "000";
signal st1N : std_logic_vector(2 downto 0);

begin

process(clk, rst, st1N)
	begin
		if(rst = '1')then
			st1 <= "000";
		else
			if(clk'event and clk = '1')then
				st1 <= st1N;
			end if;
		end if;
end process;
------------------------------------------------
process (rdwr, match, vbit, dbit, readybit,st1)

	begin 
	
		if (st1 = "000") then -- IDLE STATE
			if (rdwr = '0' and readybit = '1') then 
				st1N <= "001"; -- read
			elsif (rdwr = '1' and readybit = '1') then 
				st1N <= "010"; -- write 
			end if; 
		
		elsif (st1 = "001") then -- Read 
			if (match = '1' and vbit = '1') then 
				st1N  <= "000"; -- back to idle
			elsif (match = '0' and dbit = '0') then 
				st1N <= "011"; -- read miss d bit = 0
			elsif (match = '0' and dbit = '1') then 
				st1N <= "111"; -- write back old block to main mem since d bit = 1
			end if;
		
		elsif (st1 = "010") then -- write 
			if (match = '1' and vbit = '1') then 
				st1N <= "000"; -- back to idle
			elsif (match = '0' and dbit = '0') then
				st1N <= "101"; -- write miss d bit = 0
			elsif (match = '0' and dbit = '1') then
				st1N <= "111"; -- write back old block to main mem since d bit = 1
			end if;
			
		elsif (st1 = "011") then --read miss d bit = 0
			st1N <= "001"; -- back to read to finish reading process 
		
		elsif (st1 = "100") then --read miss d bit = 1; retrieve new block 
			st1N <= "001"; -- back to read to finish reading process 
			
		elsif (st1 = "101") then --write miss d bit = 0
			st1N <= "010"; -- back to write to finish writing process 
			
		elsif (st1 = "110") then --write miss d bit = 1; retrieve new block 
			st1N <= "010"; -- back to write to finish writing process 
			
		elsif (st1 = "111") then --write back to main memory 
			if (rdwr = '0') then 
				st1N <= "100"; -- go to retrieve new block state for read
			elsif (rdwr = '1') then
				st1N <= "110"; -- go to retrieve new block state for write
			end if; 
			
		else 
			st1N <= "000"; -- all other cases go back to Idle
		
	   end if;
	end process; 
-----------------------------------------------------------------------------

process (st1)

begin 

	if (st1 = "000") then -- IDLE STATE 
		readyOut <= '1'; -- Signalling that controller is ready to accept transactions 
	
	elsif (st1 = "001") then -- READ HIT STATE 
		readyOut <= '0'; -- signal that controller is busy
		demuxSel <= '1'; 
		
	elsif (st1 = "010") then -- WRITE HIT STATE 
		readyOut <= '0'; -- signal that controller is busy
		muxSel <= '0'; -- Data comes from CPU
      wenCache <= '1'; -- Cache is being written to 
		vbitOut <= '1'; -- set valid bit 
		dbitOut <='1'; -- set dirty bit
		DEnable <= '1'; -- enable writing to d bit reg file to update d bit 
		VEnable <='1'; -- enable writing to v bit reg file to update v bit
		
	elsif (st1 = "011") then -- READ MISS DIRTY BIT = 0
		readyOut <= '0'; -- signal controller is busy
		rdwrOut <= '0';
		wenCache <= '1';
		muxSel <= '1';
		TagEnable <= '1';
		VEnable <='1';
		vbitOut <= '1';
		mstrb <= '1';
		oldMuxSel <= '0';

   elsif (st1 = "100") then -- READ MISS D BIT = 1
		readyOut <= '0'; -- signal controller is busy
		rdwrOut <= '0';
		wenCache <= '1';
		muxSel <= '1';
		TagEnable <= '1';
		VEnable <='1';
		vbitOut <= '1';
		mstrb <= '1';
		oldMuxSel <= '0';
		
	elsif (st1 = "101") then -- WRITE D BIT = 0
		readyOut <= '0'; -- signal controller is busy
		rdwrOut <= '0';
		wenCache <= '1';
		muxSel <= '1';
		TagEnable <= '1';
		VEnable <='1';
		vbitOut <= '1';
		mstrb <= '1';
		oldMuxSel <= '0';
		
	elsif (st1 = "110") then -- WRITE D BIT = 1
		readyOut <= '0'; -- signal controller is busy
		rdwrOut <= '0';
		wenCache <= '1';
		muxSel <= '1';
		TagEnable <= '1';
		VEnable <='1';
		vbitOut <= '1';
		mstrb <= '1';
		oldMuxSel <= '0';
		
	elsif (st1 = "111") then -- WRITE BACK TO MAIN MEM
		readyOut <= '0'; -- signal controller is busy
		rdwrOut <= '1';
		wenCache <= '0';
		mstrb <= '1';
		oldMuxSel <= '1';


	end if;
	
end process;




end Behavioral;

